@@ -5,6 +5,7 @@ use crate::value::Value;
 use lazy_static::lazy_static;
 use std::cmp::Ordering;
 use std::collections::HashMap;
+use std::ops::Rem;
 
 type Function = fn(&[Value]) -> Value;
 
@@ -70,6 +71,7 @@ lazy_static! {
         map.insert("atan", numeric_atan);
         map.insert("atn2", numeric_atn2);
         map.insert("sign", numeric_sign);
+        map.insert("mod", numeric_mod);
 
         // Other Functions
         map.insert("isnull", general_is_null);
@@ -358,8 +360,8 @@ lazy_static! {
         map.insert(
             "round",
             Prototype {
-                parameters: vec![DataType::Float],
-                result: DataType::Integer,
+                parameters: vec![DataType::Float, DataType::Optional(Box::new(DataType::Integer))],
+                result: DataType::Float,
             },
         );
         map.insert(
@@ -425,6 +427,13 @@ lazy_static! {
                 result: DataType::Integer,
             },
         );
+        map.insert(
+            "mod",
+            Prototype {
+                parameters: vec![DataType::Integer, DataType::Integer],
+                result: DataType::Integer,
+            },
+        );
         // General functions
         map.insert(
             "isnull",
@@ -788,8 +797,15 @@ fn numeric_floor(inputs: &[Value]) -> Value {
 }
 
 fn numeric_round(inputs: &[Value]) -> Value {
-    let float_value = inputs[0].as_float();
-    Value::Integer(float_value.round() as i64)
+    let number = inputs[0].as_float();
+    let decimal_places = if inputs.len() == 2 {
+        inputs[1].as_int()
+    } else {
+        0
+    };
+    let multiplier = 10_f64.powi(decimal_places.try_into().unwrap());
+    let result = (number * multiplier).round() / multiplier;
+    Value::Float(result)
 }
 
 fn numeric_square(inputs: &[Value]) -> Value {
@@ -850,6 +866,16 @@ fn numeric_sign(inputs: &[Value]) -> Value {
     }
 }
 
+fn numeric_mod(inputs: &[Value]) -> Value {
+    let other = &inputs[1];
+    if other.as_int() == 0 {
+        return Value::Null;
+    }
+
+    let first = &inputs[0];
+    Value::Integer(first.as_int().rem(other.as_int()))
+}
+
 // General functions
 
 fn general_is_null(inputs: &[Value]) -> Value {
