@@ -1,5 +1,4 @@
 use gitql_ast::environment::Environment;
-use gitql_ast::environment::TABLES_FIELDS_NAMES;
 use gitql_ast::value::Value;
 use std::collections::HashMap;
 use std::num::IntErrorKind;
@@ -23,7 +22,6 @@ use gitql_ast::function::FUNCTIONS;
 use gitql_ast::function::PROTOTYPES;
 use gitql_ast::statement::*;
 use gitql_ast::types::DataType;
-use gitql_ast::types::TABLES_FIELDS_TYPES;
 
 pub fn parse_gql(tokens: Vec<Token>, env: &mut Environment) -> Result<Query, Box<Diagnostic>> {
     let mut position = 0;
@@ -55,7 +53,7 @@ pub fn parse_gql(tokens: Vec<Token>, env: &mut Environment) -> Result<Query, Box
 
 fn parse_set_query(
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Query, Box<Diagnostic>> {
     let len = tokens.len();
@@ -111,7 +109,7 @@ fn parse_set_query(
 
 fn parse_select_query(
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Query, Box<Diagnostic>> {
     let len = tokens.len();
@@ -299,7 +297,7 @@ fn parse_select_query(
 fn parse_select_statement(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     // Consume select keyword
@@ -416,14 +414,14 @@ fn parse_select_statement(
         *position += 1;
 
         table_name = &table_name_token.ok().unwrap().literal;
-        if !TABLES_FIELDS_NAMES.contains_key(table_name) {
+        if !env.schema.tables_fields_names.contains_key(table_name) {
             return Err(Diagnostic::error("Unresolved table name")
                 .add_help("Check the documentations to see available tables")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed());
         }
 
-        register_current_table_fields_types(table_name, env);
+        register_current_table_fields_types(env, table_name);
     }
 
     // Make sure `SELECT *` used with specific table
@@ -448,6 +446,7 @@ fn parse_select_statement(
     // If it `select *` make all table fields selectable
     if is_select_all {
         select_all_table_fields(
+            env,
             table_name,
             &mut context.selected_fields,
             &mut fields_names,
@@ -470,7 +469,7 @@ fn parse_select_statement(
 fn parse_where_statement(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
@@ -516,7 +515,7 @@ fn parse_where_statement(
 fn parse_group_by_statement(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
@@ -554,7 +553,7 @@ fn parse_group_by_statement(
 fn parse_having_statement(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
@@ -587,7 +586,7 @@ fn parse_having_statement(
 }
 
 fn parse_limit_statement(
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
@@ -629,7 +628,7 @@ fn parse_limit_statement(
 }
 
 fn parse_offset_statement(
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
@@ -672,7 +671,7 @@ fn parse_offset_statement(
 fn parse_order_by_statement(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     // Consume `ORDER` keyword
@@ -725,7 +724,7 @@ fn parse_order_by_statement(
 fn parse_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let aggregations_count_before = context.aggregations.len();
@@ -754,7 +753,7 @@ fn parse_expression(
 fn parse_assignment_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_is_null_expression(context, env, tokens, position)?;
@@ -791,7 +790,7 @@ fn parse_assignment_expression(
 fn parse_is_null_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_in_expression(context, env, tokens, position)?;
@@ -832,7 +831,7 @@ fn parse_is_null_expression(
 fn parse_in_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_between_expression(context, env, tokens, position)?;
@@ -913,7 +912,7 @@ fn parse_in_expression(
 fn parse_between_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_logical_or_expression(context, env, tokens, position)?;
@@ -972,7 +971,7 @@ fn parse_between_expression(
 fn parse_logical_or_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_logical_and_expression(context, env, tokens, position);
@@ -1016,7 +1015,7 @@ fn parse_logical_or_expression(
 fn parse_logical_and_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_bitwise_or_expression(context, env, tokens, position);
@@ -1060,7 +1059,7 @@ fn parse_logical_and_expression(
 fn parse_bitwise_or_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_logical_xor_expression(context, env, tokens, position);
@@ -1104,7 +1103,7 @@ fn parse_bitwise_or_expression(
 fn parse_logical_xor_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_bitwise_and_expression(context, env, tokens, position);
@@ -1146,7 +1145,7 @@ fn parse_logical_xor_expression(
 fn parse_bitwise_and_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_equality_expression(context, env, tokens, position);
@@ -1188,7 +1187,7 @@ fn parse_bitwise_and_expression(
 fn parse_equality_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_comparison_expression(context, env, tokens, position);
@@ -1252,7 +1251,7 @@ fn parse_equality_expression(
 fn parse_comparison_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_bitwise_shift_expression(context, env, tokens, position);
@@ -1317,7 +1316,7 @@ fn parse_comparison_expression(
 fn parse_bitwise_shift_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let mut lhs = parse_term_expression(context, env, tokens, position)?;
@@ -1357,7 +1356,7 @@ fn parse_bitwise_shift_expression(
 fn parse_term_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let mut lhs = parse_factor_expression(context, env, tokens, position)?;
@@ -1414,7 +1413,7 @@ fn parse_term_expression(
 fn parse_factor_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_like_expression(context, env, tokens, position);
@@ -1462,7 +1461,7 @@ fn parse_factor_expression(
 fn parse_like_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_glob_expression(context, env, tokens, position);
@@ -1506,7 +1505,7 @@ fn parse_like_expression(
 fn parse_glob_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_unary_expression(context, env, tokens, position);
@@ -1550,7 +1549,7 @@ fn parse_glob_expression(
 fn parse_unary_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     if *position < tokens.len() && is_prefix_unary_operator(&tokens[*position]) {
@@ -1589,7 +1588,7 @@ fn parse_unary_expression(
 fn parse_function_call_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_primary_expression(context, env, tokens, position)?;
@@ -1611,7 +1610,10 @@ fn parse_function_call_expression(
             let mut arguments = parse_arguments_expressions(context, env, tokens, position)?;
             let prototype = PROTOTYPES.get(function_name.as_str()).unwrap();
             let parameters = &prototype.parameters;
-            let return_type = prototype.result.clone();
+            let mut return_type = prototype.result.clone();
+            if let DataType::Dynamic(calculate_type) = return_type {
+                return_type = calculate_type(parameters);
+            }
 
             check_function_call_arguments(
                 env,
@@ -1636,7 +1638,10 @@ fn parse_function_call_expression(
             let mut arguments = parse_arguments_expressions(context, env, tokens, position)?;
             let prototype = AGGREGATIONS_PROTOS.get(function_name.as_str()).unwrap();
             let parameters = &vec![prototype.parameter.clone()];
-            let return_type = prototype.result.clone();
+            let mut return_type = prototype.result.clone();
+            if let DataType::Dynamic(calculate_type) = return_type {
+                return_type = calculate_type(parameters);
+            }
 
             check_function_call_arguments(
                 env,
@@ -1686,7 +1691,7 @@ fn parse_function_call_expression(
 fn parse_arguments_expressions(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Vec<Box<dyn Expression>>, Box<Diagnostic>> {
     let mut arguments: Vec<Box<dyn Expression>> = vec![];
@@ -1727,7 +1732,7 @@ fn parse_arguments_expressions(
 fn parse_primary_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     if *position >= tokens.len() {
@@ -1810,7 +1815,7 @@ fn parse_primary_expression(
 fn parse_group_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     *position += 1;
@@ -1828,7 +1833,7 @@ fn parse_group_expression(
 fn parse_case_expression(
     context: &mut ParserContext,
     env: &mut Environment,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let mut conditions: Vec<Box<dyn Expression>> = vec![];
@@ -1947,8 +1952,8 @@ fn parse_case_expression(
 
 fn check_function_call_arguments(
     env: &mut Environment,
-    arguments: &mut Vec<Box<dyn Expression>>,
-    parameters: &Vec<DataType>,
+    arguments: &mut [Box<dyn Expression>],
+    parameters: &[DataType],
     function_name: String,
     location: Location,
 ) -> Result<(), Box<Diagnostic>> {
@@ -2080,7 +2085,7 @@ fn type_check_selected_fields(
     env: &mut Environment,
     table_name: &str,
     fields_names: &Vec<String>,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: usize,
 ) -> Result<(), Box<Diagnostic>> {
     for field_name in fields_names {
@@ -2123,7 +2128,7 @@ fn un_expected_statement_error(tokens: &[Token], position: &mut usize) -> Box<Di
         .as_boxed()
 }
 
-fn un_expected_expression_error(tokens: &Vec<Token>, position: &usize) -> Box<Diagnostic> {
+fn un_expected_expression_error(tokens: &[Token], position: &usize) -> Box<Diagnostic> {
     let location = get_safe_location(tokens, *position);
 
     if *position == 0 || *position >= tokens.len() {
@@ -2199,7 +2204,7 @@ fn un_expected_expression_error(tokens: &Vec<Token>, position: &usize) -> Box<Di
 /// Report error message for extra content after the end of current statement
 fn un_expected_content_after_correct_statement(
     statement_name: &str,
-    tokens: &Vec<Token>,
+    tokens: &[Token],
     position: &mut usize,
 ) -> Box<Diagnostic> {
     let error_message = &format!(
@@ -2237,23 +2242,24 @@ fn get_expression_name(expression: &Box<dyn Expression>) -> Result<String, ()> {
 }
 
 #[inline(always)]
-fn register_current_table_fields_types(table_name: &str, symbol_table: &mut Environment) {
-    let table_fields_names = &TABLES_FIELDS_NAMES[table_name];
+fn register_current_table_fields_types(env: &mut Environment, table_name: &str) {
+    let table_fields_names = &env.schema.tables_fields_names[table_name].clone();
     for field_name in table_fields_names {
-        let field_type = TABLES_FIELDS_TYPES[field_name].clone();
-        symbol_table.define(field_name.to_string(), field_type);
+        let field_type = env.schema.tables_fields_types[field_name].clone();
+        env.define(field_name.to_string(), field_type);
     }
 }
 
 #[inline(always)]
 fn select_all_table_fields(
+    env: &mut Environment,
     table_name: &str,
     selected_fields: &mut Vec<String>,
     fields_names: &mut Vec<String>,
     fields_values: &mut Vec<Box<dyn Expression>>,
 ) {
-    if TABLES_FIELDS_NAMES.contains_key(table_name) {
-        let table_fields = &TABLES_FIELDS_NAMES[table_name];
+    if env.schema.tables_fields_names.contains_key(table_name) {
+        let table_fields = &env.schema.tables_fields_names[table_name];
 
         for field in table_fields {
             if !fields_names.contains(&field.to_string()) {
@@ -2271,7 +2277,7 @@ fn select_all_table_fields(
 }
 
 #[inline(always)]
-fn consume_kind(tokens: &Vec<Token>, position: usize, kind: TokenKind) -> Result<&Token, ()> {
+fn consume_kind(tokens: &[Token], position: usize, kind: TokenKind) -> Result<&Token, ()> {
     if position < tokens.len() && tokens[position].kind == kind {
         return Ok(&tokens[position]);
     }
@@ -2279,7 +2285,7 @@ fn consume_kind(tokens: &Vec<Token>, position: usize, kind: TokenKind) -> Result
 }
 
 #[inline(always)]
-fn get_safe_location(tokens: &Vec<Token>, position: usize) -> Location {
+fn get_safe_location(tokens: &[Token], position: usize) -> Location {
     if position < tokens.len() {
         return tokens[position].location;
     }
